package com.lantone.security.component;

import com.lantone.dblayermbg.facade.DictionaryInfoFacade;
import com.lantone.dblayermbg.facade.HospitalSetFacade;
import com.lantone.dblayermbg.facade.ResourceFacade;
import com.lantone.security.aop.DictionaryAspect;
import com.lantone.security.aop.HospitalSetAspect;
import com.lantone.security.aop.ResourceRoleRulesAspect;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @Description: 基础信息变更导致redis需要刷新-消息的处理者
 * @author: rengb
 * @time: 2021/1/5 18:27
 */
@Slf4j
@Component
public class MessageReceiver {

    @Autowired
    private DictionaryInfoFacade dictionaryInfoFacade;
    @Autowired
    private HospitalSetFacade hospitalSetFacade;
    @Autowired
    private ResourceFacade resourceFacade;

    @RabbitListener(queues = "emrais.redis.cache.refresh.queue", ackMode = "MANUAL")
    public void RedisCacheRefreshHandle(String type, Message message, Channel channel) throws Exception {
        if (type.equals(DictionaryAspect.class.getSimpleName())) {
            dictionaryInfoFacade.initDicMap();
        } else if (type.equals(HospitalSetAspect.class.getSimpleName())) {
            hospitalSetFacade.initHospitalSetMap();
        } else if (type.equals(ResourceRoleRulesAspect.class.getSimpleName())) {
            resourceFacade.initResourceRolesMap();
        }
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
    }

}