package com.lantone.security.config;

import com.lantone.security.enums.QueueEnum;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.ExchangeBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @Description: 消息队列相关配置
 * @author: rengb
 * @time: 2021/1/5 18:27
 */
@Configuration
public class RabbitMqConfig {

    /**
     * 基础信息变更导致redis需要刷新—队列所绑定的交换机
     */
    @Bean
    DirectExchange redisCacheRefreshDirect() {
        return ExchangeBuilder.directExchange(QueueEnum.QUEUE_REDIS_CACHE_REFRESH.getExchange()).durable(true).build();
    }

    /**
     * 基础信息变更导致redis需要刷新—队列
     */
    @Bean
    public Queue redisCacheRefreshQueue() {
        return new Queue(QueueEnum.QUEUE_REDIS_CACHE_REFRESH.getName(), true, false, false);
    }


    /**
     * 将-基础信息变更导致redis需要刷新-队列绑定到交换机
     */
    @Bean
    Binding redisCacheRefreshBinding(DirectExchange redisCacheRefreshDirect, Queue redisCacheRefreshQueue) {
        return BindingBuilder
                .bind(redisCacheRefreshQueue)
                .to(redisCacheRefreshDirect)
                .with(QueueEnum.QUEUE_REDIS_CACHE_REFRESH.getRouteKey());
    }

}